@@ -1,6 +1,6 @@
 import json
 import os
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Literal
 
 import pandas as pd
 from prefect import Flow, Parameter, task
@@ -74,20 +74,26 @@ def df_to_csv_task(df, path: str, sep: str = "\t"):
     df.to_csv(path, sep=sep, index=False)
 
 
+@task
+def df_to_parquet_task(df, path: str):
+    df.to_parquet(path)
+
+
 class ADLSToAzureSQL(Flow):
     def __init__(
         self,
         name: str,
         local_file_path: str = None,
         adls_path: str = None,
-        sep: str = "\t",
+        read_sep: str = "\t",
+        write_sep: str = "\t",
         overwrite_adls: bool = True,
         if_empty: str = "warn",
         adls_sp_credentials_secret: str = None,
         dtypes: Dict[str, Any] = None,
         table: str = None,
         schema: str = None,
-        if_exists: str = "replace",  # this applies to the full CSV file, not per chunk
+        if_exists: Literal["fail", "replace", "append"] = "replace",
         sqldb_credentials_secret: str = None,
         max_download_retries: int = 5,
         tags: List[str] = ["promotion"],
@@ -105,7 +111,8 @@ class ADLSToAzureSQL(Flow):
             local_file_path (str, optional): Local destination path. Defaults to None.
             adls_path (str): The path to an ADLS folder or file. If you pass a path to a directory,
             the latest file from that directory will be loaded. We assume that the files are named using timestamps.
-            sep (str, optional): The separator to use in the CSV. Defaults to "\t".
+            read_sep (str, optional): The delimiter for the source file. Defaults to "\t".
+            write_sep (str, optional): The delimiter for the output CSV file. Defaults to "\t".
             overwrite_adls (bool, optional): Whether to overwrite the file in ADLS. Defaults to True.
             if_empty (str, optional): What to do if the Supermetrics query returns no data. Defaults to "warn".
             adls_sp_credentials_secret (str, optional): The name of the Azure Key Vault secret containing a dictionary with
@@ -115,7 +122,7 @@ class ADLSToAzureSQL(Flow):
             To be used only in case that dtypes need to be manually mapped - dtypes from raw schema file in use by default. Defaults to None.
             table (str, optional): Destination table. Defaults to None.
             schema (str, optional): Destination schema. Defaults to None.
-            if_exists (str, optional): What to do if the table exists. Defaults to "replace".
+            if_exists (Literal, optional): What to do if the table exists. Defaults to "replace".
             sqldb_credentials_secret (str, optional): The name of the Azure Key Vault secret containing a dictionary with
             Azure SQL Database credentials. Defaults to None.
             max_download_retries (int, optional): How many times to retry the download. Defaults to 5.
@@ -123,14 +130,13 @@ class ADLSToAzureSQL(Flow):
             vault_name (str, optional): The name of the vault from which to obtain the secrets. Defaults to None.
         """
         adls_path = adls_path.strip("/")
-
-        # Read parquet file from RAW
+        # Read parquet
         if adls_path.split(".")[-1] in ["csv", "parquet"]:
             self.adls_path = adls_path
         else:
             self.adls_path = get_key_value(key=adls_path)
 
-        # Read schema info json from RAW
+        # Read schema
         self.dtypes = dtypes
         self.adls_root_dir_path = os.path.split(self.adls_path)[0]
         self.adls_file_name = os.path.split(self.adls_path)[-1]
@@ -145,17 +151,20 @@ class ADLSToAzureSQL(Flow):
         # AzureDataLakeUpload
         self.local_file_path = local_file_path or self.slugify(name) + ".csv"
         self.local_json_path = self.slugify(name) + ".json"
-        self.sep = sep
+        self.read_sep = read_sep
+        self.write_sep = write_sep
         self.overwrite_adls = overwrite_adls
         self.if_empty = if_empty
         self.adls_sp_credentials_secret = adls_sp_credentials_secret
         self.adls_path_conformed = self.get_promoted_path(env="conformed")
         self.adls_path_operations = self.get_promoted_path(env="operations")
 
-        # BCPTask
+        # AzureSQLCreateTable
         self.table = table
         self.schema = schema
-        self.if_exists = if_exists
+        self.if_exists = self._map_if_exists(if_exists)
+
+        # BCPTask
         self.sqldb_credentials_secret = sqldb_credentials_secret
 
         # Global
@@ -168,29 +177,51 @@ class ADLSToAzureSQL(Flow):
         # self.dtypes.update(METADATA_COLUMNS)
         self.gen_flow()
 
+    @staticmethod
+    def _map_if_exists(if_exists: str) -> str:
+        mapping = {"append": "skip"}
+        return mapping.get(if_exists, if_exists)
+
     @staticmethod
     def slugify(name):
         return name.replace(" ", "_").lower()
 
     def get_promoted_path(self, env: str) -> str:
         adls_path_clean = self.adls_path.strip("/")
-        extension = adls_path_clean.split(".")[-1]
-        if extension in ["csv", "parquet"]:
+        extension = adls_path_clean.split(".")[-1].strip()
+        if extension == "parquet":
             file_name = adls_path_clean.split("/")[-2] + ".csv"
             common_path = "/".join(adls_path_clean.split("/")[1:-2])
         else:
-            file_name = adls_path_clean.split("/")[-1] + ".csv"
+            file_name = adls_path_clean.split("/")[-1]
             common_path = "/".join(adls_path_clean.split("/")[1:-1])
 
         promoted_path = os.path.join(env, common_path, file_name)
 
         return promoted_path
 
+    def create_to_file_task(self, df, file_type):
+        df_to_type = None
+        if file_type == "csv":
+            df_to_type = df_to_csv_task.bind(
+                df=df,
+                path=self.local_file_path,
+                sep=self.write_sep,
+                flow=self,
+            )
+        else:
+            df_to_type = df_to_parquet_task.bind(
+                df=df,
+                path=self.local_file_path,
+                flow=self,
+            )
+        return df_to_type
+
     def gen_flow(self) -> Flow:
-        adls_raw_file_path = Parameter("adls_raw_file_path", default=self.adls_path)
         df = lake_to_df_task.bind(
-            path=adls_raw_file_path,
+            path=self.adls_path,
             sp_credentials_secret=self.adls_sp_credentials_secret,
+            sep=self.read_sep,
             flow=self,
         )
 
@@ -206,12 +237,9 @@ class ADLSToAzureSQL(Flow):
         else:
             dtypes = self.dtypes
 
-        df_to_csv = df_to_csv_task.bind(
-            df=df,
-            path=self.local_file_path,
-            sep=self.sep,
-            flow=self,
-        )
+        adls_file_type = self.adls_path.split(".")[-1]
+        df_to_type = self.create_to_file_task(df, adls_file_type)
+
         promote_to_conformed_task.bind(
             from_path=self.local_file_path,
             to_path=self.adls_path_conformed,
@@ -246,8 +274,8 @@ class ADLSToAzureSQL(Flow):
         )
 
         # dtypes.set_upstream(download_json_file_task, flow=self)
-        promote_to_conformed_task.set_upstream(df_to_csv, flow=self)
+        promote_to_conformed_task.set_upstream(df_to_type, flow=self)
         # map_data_types_task.set_upstream(download_json_file_task, flow=self)
-        create_table_task.set_upstream(df_to_csv, flow=self)
+        create_table_task.set_upstream(df_to_type, flow=self)
         promote_to_operations_task.set_upstream(promote_to_conformed_task, flow=self)
         bulk_insert_task.set_upstream(create_table_task, flow=self)
